#ifndef _IDSA_OUTPUT_
#define _IDSA_OUTPUT_

#define OUTPUT_TYPE_NULL    0x00
#define OUTPUT_TYPE_FILE    0x01
#define OUTPUT_TYPE_PIPE    0x02

#define OUTPUT_FORMAT_BINARY  0x01 /* ugly binary format */
#define OUTPUT_FORMAT_ULM     0x02 /* bishop, abela et al had a good idea */
#define OUTPUT_FORMAT_TULM    0x03 /* ULM + type information */
#define OUTPUT_FORMAT_NATIVE  0x04 /* my native format */
#define OUTPUT_FORMAT_SYSLOG  0x05 /* the old system logger */
#define OUTPUT_FORMAT_CSV     0x06 /* comma separated values */

#define OUTPUT_FLAG_ALIAS   0x01 /* occurs more than once, unused */
#define OUTPUT_FLAG_SYNC    0x02 /* open file sync or wait for pipe */
#define OUTPUT_FLAG_SEEK    0x04 /* can do seek back */
#define OUTPUT_FLAG_REPEAT  0x10 /* print repeat count instead of dup message */
#define OUTPUT_FLAG_ROTATE  0x20 /* automatically rotate logs */
#define OUTPUT_FLAG_REPORT  0x40 /* report logging failures */

#include <unistd.h>

#include <idsa_internal.h>

#include "idsad.h"

struct log_output{
  int o_fd,o_xd; /* file descriptors, xd for rotate  */
  int o_type; /* output type */
  int o_flags; /* flags */

  IDSA_LOOKUP_OUTPUT *o_format;

  char o_name[IDSA_M_MEDIUM]; /* name of destinantion */
  pid_t o_pid; /* pid in case of pipe */

  /* valid if OUTPUT_FLAG_ROTATE  and OUTPUT_TYPE_FILE */
  int o_sizemax; /* size above which we rotate */
  int o_sizehave; /* current size */

  /* valid if OUTPUT_FLAG_REPEAT */
  int o_repfactor; /* multiply target by this to get new target */
  int o_reptarget; /* log if we reach this repeat count */
  int o_repcount; /* current number of repeats */
  IDSA_EVENT *o_repevent; /* need event to check for repeats */

  /* valid of OUTPUT_FLAG_SEEK */
  int o_seekback;

  int o_dropcount; /* count of events dropped to the floor */
  int o_errno; /* reason for dropping things (disk full, pipe broken) */
  int o_fresh; /* recent error */

  struct log_output *o_next;
};

LOG_OUTPUT *output_insert(LOG_OUTPUT *o, char *s);
int output_free(LOG_OUTPUT *o);

void output_dump(LOG_OUTPUT *o, FILE *f);
void output_start(STATE_SET *s, LOG_OUTPUT *o);

void output_clear(LOG_OUTPUT *o);
int output_run(LOG_OUTPUT *o, IDSA_EVENT *e);
void output_error(STATE_SET *s);

#endif
